// 3DTEST.CPP: Tests the TThreeD class by displaying a
// wire-frame view of an object in a window. The file
// displayed has the name TEST1.DAT.

#include <owl.h>
#include <stdio.h>
#include "threed.h"

#define FILENAME "TEST1.DAT"

class TThreeDApp : public TApplication {
public:
  TThreeDApp(LPSTR AName, HANDLE hInstance, HANDLE hPrevInstance,
    LPSTR lpCmdLine, int nCmdShow)
    : TApplication(AName, hInstance, hPrevInstance, lpCmdLine, nCmdShow) { };
  virtual void Error(int ErrorCode);
  virtual void InitMainWindow();
};

class TThreeDWindow : public TWindow {
  PTThreeD ThreeD;
public:
  TThreeDWindow(PTWindowsObject AParent, LPSTR ATitle);
  ~TThreeDWindow();
  virtual void Paint(HDC hDC, PAINTSTRUCT&);
};

TThreeDWindow::TThreeDWindow(PTWindowsObject AParent, LPSTR ATitle)
  : TWindow(AParent, ATitle)
{
  int Temp;
  ThreeD = new TThreeD;   // Create a TThreeD object
  if ((Temp=ThreeD->Read3DObject(FILENAME)) == TRUE) {
    ThreeD->SetAt();
    ThreeD->SetFrom();
  }
  else
    // Status, defined by OWL, is used to report errors. It
    // will pop up an appropriate message. See TThreeDApp::Error().
    Status = Temp;
}

TThreeDWindow::~TThreeDWindow()
{
  delete ThreeD;
}

void TThreeDWindow::Paint(HDC hDC, PAINTSTRUCT&)
{
  RECT rect;
  GetClientRect(HWindow, &rect);
  ThreeD->Display(hDC, rect);
}

void TThreeDApp::Error(int ErrorCode)
{
  char ErrorString[51];

  switch(ErrorCode) {
    case EM_FILEOPENERROR:
      MessageBox(0, "Could not open file", Name, MB_OK);
      return;
    case EM_FILETOOBIG:
      MessageBox(0, "File's objects are too big", Name, MB_OK);
      return;
    case EM_OUTOFMEMORY:
      sprintf(ErrorString, "Out of memory");
      break;
    default:
      sprintf(ErrorString,
	"Error received: error code = %d\nExit Program ?", ErrorCode);
      break;
  }
  // Show the error message and ask the user if they want to quit
  // the program
  if (MessageBox(0, ErrorString, Name, MB_ICONSTOP | MB_YESNO) == IDYES)
    exit(ErrorCode);
}

void TThreeDApp::InitMainWindow()
{
  MainWindow = new TThreeDWindow(NULL, "3DTEST");
}

int PASCAL WinMain(HANDLE hInstance, HANDLE hPrevInstance,
  LPSTR lpCmdLine, int nCmdShow)
{
  TThreeDApp ThreeDApp("3DTEST", hInstance, hPrevInstance,
    lpCmdLine, nCmdShow);
  ThreeDApp.Run();
  return ThreeDApp.Status;
}

